/*
 * Copyright 2015 JBoss, by Red Hat, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.uberfire.ext.security.server;

import javax.enterprise.event.Observes;
import javax.enterprise.inject.spi.AnnotatedType;
import javax.enterprise.inject.spi.BeanManager;
import javax.enterprise.inject.spi.Extension;

public class HttpServletRequestProducerExtension implements Extension {

    private boolean isHttpRequestSupportAvailable = false;

    public HttpServletRequestProducerExtension() {
        try {
            Class.forName("javax.enterprise.inject.spi.CDI");
            isHttpRequestSupportAvailable = true;
        } catch (ClassNotFoundException e) {
            isHttpRequestSupportAvailable = false;
        }
    }

    <X> void processAnnotatedType(@Observes final javax.enterprise.inject.spi.ProcessAnnotatedType<X> pat,
                                  BeanManager beanManager) {
        if (isHttpRequestSupportAvailable) {

            final AnnotatedType<X> annotatedType = pat.getAnnotatedType();
            final Class<X> javaClass = annotatedType.getJavaClass();

            if (javaClass != null && javaClass.getName().equals(HttpRequestProducer.class.getName())) {
                pat.veto();
            }
        }
        return;
    }
}
